# Generated by Django 4.0.10 on 2023-06-03 14:54

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveuniverse", "0009_load_industry_activities"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eveindustryactivityduration",
            name="eve_type",
            field=models.ForeignKey(
                help_text="Blueprint type",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="industry_durations",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="eveindustryactivitymaterial",
            name="eve_type",
            field=models.ForeignKey(
                help_text="Blueprint type",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="industry_materials",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="eveindustryactivityproduct",
            name="eve_type",
            field=models.ForeignKey(
                help_text="Blueprint type",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="industry_products",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="eveindustryactivityskill",
            name="eve_type",
            field=models.ForeignKey(
                help_text="Blueprint type",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="industry_skills",
                to="eveuniverse.evetype",
            ),
        ),
    ]
