# Generated by Django 4.0.3 on 2022-03-05 20:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveuniverse", "0005_type_materials_and_sections"),
    ]

    operations = [
        migrations.AlterField(
            model_name="evedogmaeffectmodifier",
            name="operator",
            field=models.IntegerField(default=None, null=True),
        ),
    ]
