# Generated by Django 4.0.3 on 2022-04-11 16:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveuniverse", "0006_alter_evedogmaeffectmodifier_operator"),
    ]

    operations = [
        migrations.AddField(
            model_name="evetype",
            name="description",
            field=models.TextField(default=""),
        ),
    ]
